package com.translogicsys.aspect;

import java.util.*;
import java.io.IOException;
import org.aspectj.lang.reflect.*;
import org.aspectj.lang.*;


aspect SampleAspect
{

	pointcut allCallsWithNFEThrowsClause(): execution(* *.*(..) throws NumberFormatException) &&
	!within(com.translogicsys.aspect.MockTestCase);

	Object around() throws NumberFormatException: allCallsWithNFEThrowsClause()
	{
		String className = thisJoinPoint.getSignature().getDeclaringType().getName();
		Object receiver = thisJoinPoint.getThis();
		if (receiver != null)
			className = receiver.getClass().getName();
		String methodName = thisJoinPoint.getSignature().getName();
        System.out.println("allCallsWithNFEThrowsClause() is called: " + className + "." + methodName);
		Object returnValue = MockTestCase.getMockReturnValue(className, methodName);
		if(returnValue != null) {
			return returnValue;
		}

		NumberFormatException nfex = null;
		try {
			nfex = (NumberFormatException) MockTestCase.getMockException(className,methodName);
	        System.out.println("allCallsWithNFEThrowsClause() is called and exception is raised: ex:" + nfex);
			MockTestCase.indicateCalled(className, methodName, getArguments(thisJoinPoint));
            throw nfex;
		}catch(ClassCastException ex) {
			return proceed();
		}catch(NullPointerException ex) {
			return proceed();
		}
	}
	
	pointcut allCallsWithIOEThrowsClause(): execution(* *.*(..) throws IOException) &&
	!within(com.translogicsys.aspect.MockTestCase);

	Object around() throws IOException: allCallsWithIOEThrowsClause()
	{
		String className = thisJoinPoint.getSignature().getDeclaringType().getName();
		Object receiver = thisJoinPoint.getThis();
		if (receiver != null)
			className = receiver.getClass().getName();
		String methodName = thisJoinPoint.getSignature().getName();
        System.out.println("allCallsWithIOEThrowsClause() is called: " + className + "." + methodName);
		Object returnValue = MockTestCase.getMockReturnValue(className, methodName);
		if(returnValue != null) {
			return returnValue;
		}

		IOException nfex = null;
		try {
			nfex = (IOException) MockTestCase.getMockException(className,methodName);
	        System.out.println("allCallsWithIOEThrowsClause() is called and exception is raised: ex:" + nfex);
			MockTestCase.indicateCalled(className, methodName, getArguments(thisJoinPoint));
            throw nfex;
		}catch(ClassCastException ex) {
			return proceed();
		}catch(NullPointerException ex) {
			return proceed();
		}
	}
	

	pointcut allCalls(): execution(* *.*(..)) && !within(com.translogicsys.aspect.MockTestCase);

	Object around() : allCalls()
	{
		String className = thisJoinPoint.getSignature().getDeclaringType().getName();
		Object receiver = thisJoinPoint.getThis();
		if (receiver != null)
			className = receiver.getClass().getName();
		String methodName = thisJoinPoint.getSignature().getName();
		Object returnValue = MockTestCase.getMockReturnValue(className, methodName);
		//Throwable t=MockTestCase.getMockException(className,methodName);
        System.out.println("allCalls() is called: " + className + "." + methodName);
		if (returnValue != null)
		{
			MockTestCase.indicateCalled(className, methodName,
			getArguments(thisJoinPoint));
			return returnValue;
			/*
		}else if(t!=null) {
			MockTestCase.indicateCalled(className, methodName,
            getArguments(thisJoinPoint));
            throw t;
			*/
        }
		else
		{
			return proceed();
		}
	}





	private Hashtable getArguments(JoinPoint jp)
	{
		Hashtable arguments = new Hashtable();
		Object[] argumentValues = jp.getArgs();
		String[] argumentNames =
		((CodeSignature)jp.getSignature()).getParameterNames();
		for (int i = 0; i < argumentValues.length; i++)
		{
			if (argumentValues[i] != null)
				arguments.put(argumentNames[i], argumentValues[i]);
		}
		return arguments;
	}
}

